@@ -7,17 +7,22 @@
 ## Modified for new Scythe and rngs 7/25/2004 KQ
 ## note: B0 is now a precision
 ## Modified to allow user-specified prior density 8/17/2005 KQ
+## Modified to handle marginal likelihood calculation 1/27/2006 KQ
 ##
 
 "MCMClogit" <-
   function(formula, data = parent.frame(), burnin = 1000, mcmc = 10000,
            thin=1, tune=1.1, verbose = 0, seed = NA, beta.start = NA,
-           b0 = 0, B0 = 0, user.prior.density=NULL, logfun=TRUE, ...) {
+           b0 = 0, B0 = 0, user.prior.density=NULL, logfun=TRUE,
+           marginal.likelihood = c("none", "Laplace"), ...) {
 
     ## checks
     check.offset(list(...))
     check.mcmc.parameters(burnin, mcmc, thin)
+
+    cl <- match.call()
     
+       
     ## seeds
     seeds <- form.seeds(seed) 
     lecuyer <- seeds[[1]]
@@ -37,6 +42,22 @@
     b0 <- mvn.prior[[1]]
     B0 <- mvn.prior[[2]]
 
+
+    ## get marginal likelihood argument
+    marginal.likelihood  <- match.arg(marginal.likelihood)
+    B0.eigenvalues <- eigen(B0)$values
+    if (min(B0.eigenvalues) < 0){
+      stop("B0 is not positive semi-definite.\nPlease respecify and call again.\n")
+    }
+    if (isTRUE(all.equal(min(B0.eigenvalues), 0))){
+      if (marginal.likelihood != "none"){
+        warning("Cannot calculate marginal likelihood with improper prior\n")
+        marginal.likelihood <- "none"
+      }
+    }
+    logmarglike <- NULL
+
+    
     ## setup the environment so that fun can see the things passed as ...
     userfun <- function(ttt) user.prior.density(ttt, ...)
     my.env <- environment(fun = userfun)
@@ -102,7 +123,8 @@
       sample <- matrix(data=0, mcmc/thin, dim(X)[2] )
       
       auto.Scythe.call(output.object="posterior", cc.fun.name="MCMClogit",
-                       sample=sample, Y=Y, X=X, burnin=as.integer(burnin),
+                       sample.nonconst=sample, Y=Y, X=X,
+                       burnin=as.integer(burnin),
                        mcmc=as.integer(mcmc), thin=as.integer(thin),
                        tune=tune, lecuyer=as.integer(lecuyer),
                        seedarray=as.integer(seed.array),
@@ -110,10 +132,29 @@
                        verbose=as.integer(verbose), betastart=beta.start,
                        b0=b0, B0=B0, V=V) 
 
+      ## marginal likelihood calculation if Laplace
+      if (marginal.likelihood == "Laplace"){
+        theta.start <- beta.start
+        optim.out <- optim(theta.start, logpost.logit, method="BFGS",
+                           control=list(fnscale=-1),
+                           hessian=TRUE, y=Y, X=X, b0=b0, B0=B0)
+        
+        theta.tilde <- optim.out$par
+        beta.tilde <- theta.tilde[1:K]
+        
+        Sigma.tilde <- solve(-1*optim.out$hessian)
+        
+        logmarglike <- (length(theta.tilde)/2)*log(2*pi) +
+          log(sqrt(det(Sigma.tilde))) + 
+            logpost.logit(theta.tilde, Y, X, b0, B0)
+        
+      }
+      
+      
       ## put together matrix and build MCMC object to return
       output <- form.mcmc.object(posterior, names=xnames,
-                                 title="MCMClogit Posterior Density Sample")
-      
+                                 title="MCMClogit Posterior Sample",
+                                 y=Y, call=cl, logmarglike=logmarglike)
       
     }
     else {
@@ -129,11 +170,36 @@
                       as.logical(logfun),
                       as.matrix(propvar),
                       PACKAGE="MCMCpack")
+
+      
+      ## marginal likelihood calculation if Laplace
+      if (marginal.likelihood == "Laplace"){
+        theta.start <- beta.start
+        optim.out <- optim(theta.start, logpost.logit.userprior, method="BFGS",
+                           control=list(fnscale=-1),
+                           hessian=TRUE, y=Y, X=X, userfun=userfun,
+                           logfun=logfun, my.env=my.env)
+        
+        theta.tilde <- optim.out$par
+        beta.tilde <- theta.tilde[1:K]
+        
+        Sigma.tilde <- solve(-1*optim.out$hessian)
+        
+        logmarglike <- (length(theta.tilde)/2)*log(2*pi) +
+          log(sqrt(det(Sigma.tilde))) + 
+            logpost.logit.userprior(theta.tilde, Y, X, userfun=userfun,
+                                    logfun=logfun, my.env=my.env)
+        
+      }
+
       
       output <- mcmc(data=sample, start=burnin+1,
                      end=burnin+mcmc, thin=thin)
       varnames(output) <- as.list(xnames)
-      attr(output, "title") <- "MCMClogit Posterior Density Sample"
+      attr(output, "title") <- "MCMClogit Posterior Sample"
+      attr(output, "y") <- Y
+      attr(output, "call") <- cl
+      attr(output, "logmarglike") <- logmarglike
     }
     
     return(output)    
